{-----------------------------------------------------------------------------------
Diese Unit ist Teil des Programms USBoot.

Das Programm USBoot ist ein einfaches Programm zum Laden eines HEX-Files
in einen PIC18Fxxxx mit USB-Schnittstelle und USB-Bootloader.
Es bentigt den Bootloader von:
http://sprut.de/electronic/pic/8bit/18f/programm/usbbootloader/usbboot.htm#bootloader
oder den Bootloader 1.0 aus dem USBurn-Paket:
http://sprut.de/electronic/soft/usburn/usburn.htm#download

Nach nderung des vid_pid-Wertes in usbdll.pas kann auch der Microchip USB-Bootloader verwendet werden:
http://www.microchip.com/stellent/idcplg?IdcService=SS_GET_PAGE&nodeId=2124&param=en022627


Copyright (C) 2006  Joerg Bredendiek
 Homepage: http:www.sprut.de
 E-Mail:   sprut@sprut.de

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.


Auf Deutsch:
Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen
der GNU General Public License, wie von der Free Software Foundation verffentlicht,
weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder
(nach Ihrer Option) jeder spteren Version.

Die Verffentlichung dieses Programms erfolgt in der Hoffnung, da es Ihnen von Nutzen
sein wird, aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite Garantie der MARKTREIFE
oder der VERWENDBARKEIT FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der
GNU General Public License.

Sie sollten eine Kopie der GNU General Public License zusammen mit diesem Programm erhalten haben.
Falls nicht, schreiben Sie an die Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

--------------------------------------------------------------------------------------}

unit usbboot;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Gauges, ComCtrls, ExtCtrls;

type
  // Speicher-Datenstruktur verschiedener PIC-Familien
  // Fr den Bootloader trifft "16" zu
  // Config:
  //        14:    2007..             x 14 Bit
  //        16:    300000..30000D  13 x  8 Bit
  //        24:    F80000..F8000E   8 x 16 Bit    (nur gerade Adressen; meist nur 7 x 16 Bit)
  // ID:
  //        14:    2000..2003       4 x 4 Bit (von 14 Bit) $2004 ist 14 Bit Debug
  //        16:    200000..200007   8 x 4 Bit (von  8 Bit)
  //        24:
  // FLASH:
  //        14:     0000.. 1FFF       x 14 Bit (max. 8 k Flash)
  //        16:    00000..1FFFF       x  8 Bit (max. 128 kByte)
  //        24:    00000..1FFFF       x 24 Bit (max. 48 k x 24 Bit)
  // EEPROM:
  //        14:     00.. FF       x  8 Bit      (max. 256 x 8 Bit EEPROM)
  //        16:    000..3FF       x  8 Bit
  //        24:    000..7FF       x 16 Bit      (max. 2k x 16 Bit)

  TConfig  = array[0..14] of word;          // Config
  TFlash   = array[$0000..$1FFFF] of longword;

  TPIC = record
           Flash    : TFlash;
           ROM      : array[0..$7FF] of word;
           ID       : array[0..7]    of word; // ID und debug
           Config   : TConfig;
         end;


  Tbootloader = class(TForm)
    BootGroupBox: TGroupBox;
    BootGauge: TGauge;
    Label6: TLabel;
    Memo18: TMemo;
    Button44: TButton;
    Button45: TButton;
    StatusBar1: TStatusBar;
    OpenDialog1: TOpenDialog;
    fakeb8CheckBox: TCheckBox;
    fakeBLCheckBox: TCheckBox;
    Button1: TButton;
    Button2: TButton;
    TargetRadioGroup: TRadioGroup;
    Button3: TButton;
    fakeTBCheckBox: TCheckBox;
    Button4: TButton;
    procedure EraseTPIC(var PIC:TPIC);
    procedure CheckForUsbBrenner;
    procedure LoadFile;
    procedure OpenHexfile16;
    procedure Button45Click(Sender: TObject);
    procedure Button44Click(Sender: TObject);
    procedure ActBootloader(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure fakeBLCheckBoxClick(Sender: TObject);
    procedure fakeb8CheckBoxClick(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure fakeTBCheckBoxClick(Sender: TObject);
    procedure Button4Click(Sender: TObject);
  private
    { Private-Deklarationen }
  public
    { Public-Deklarationen }
  end;

var
  bootloader: Tbootloader;
  Kern      : integer =16;
  FName     : string = '';
  hexpfad   : string = '';
  Hexfile        : text;
  FileFlag       : boolean=false;   // Hex-file geladen?
  Hexin          : TPIC;
  Maske          : TPIC;
  MaxFlash       : integer=$7FFF;    //letzte Flash-Adresse fr 18F2550 worte
  MaxEE          : integer=255;      //letzte EEPROM-Adresse
  EndFlash       : longint=0;        //letzte benutzte FlashAdresse
  EndEE          : longint=-1;       //letzte benutzte EEPROM-Adresse
  NoConfig       : boolean = false;
  BootloaderDa   : boolean = false;
  brennerda      : boolean = false;
  Firmware       : integer;
  HexConfig      : boolean = false;  //hex enthlt config
  blabla         : boolean = false;  //fr info
  


implementation

uses
  usbdll, brennusbuni;

{$R *.DFM}


//alles mit FFh beschreiben
{
  TConfig  = array[0..14] of word;          // Config
  TFlash   = array[$0000..$1FFFF] of longword;
  TPIC = record
           Flash    : TFlash;
           ROM      : array[0..$7FF] of word;
           ID       : array[0..7] of word; // ID und debug
           Config   : TConfig;
         end;
}
procedure Tbootloader.EraseTPIC(var PIC:TPIC);
var k : integer;
begin
  for k:=0     to $1FFFF do PIC.Flash[k] :=$FFFFFFFF;  //longword  32 Bit ohne Vorzeichen
  for k:=0     to   $7FF do PIC.ROM[k]   :=    $FFFF;
  for k:=0     to      7 do PIC.ID[k]    :=    $FFFF;
  for k:=0     to     14 do PIC.Config[k]:=    $FFFF;
end;   // EraseTPIC


//versuche file 'Fname' zu ffenen und zu lesen
procedure Tbootloader.LoadFile;
var
  Fehler       : integer;
  k            : integer;
begin
  //ReadFromFile;
  if FName<>'' then begin
    assignFile(Hexfile, FName);
    (*$I-*)
    reset(Hexfile);
    (*$I+*)
    Fehler:=IOResult;
    if Fehler=0 then begin
      EraseTPIC(Hexin);
      EraseTPIC(Maske);
      closefile(Hexfile);
      FileFlag:=true;
      OpenHexfile16;  //einlesen nach  hexin[]
      StatusBar1.Panels[1].Text:='';
    end else begin
      FileFlag:=False;
      StatusBar1.Panels[1].Text:='!! file load ERROR !!';
      showmessage('HEX-File  ' + FName + '  nicht gefunden');
      Memo18.Lines.Add('## HEX-file  ' + ExtractFileName(FName) + ' don''t exists');
    end;
  end;
end;   // LoadFile


(*hex file nach hexin[] lesen*)
procedure Tbootloader.OpenHexfile16;
var ch           : char;
    ByteAnzahl   : Word;
    HexTyp       : Word;
    Adresse      : longword;
    ULBA         : longword;
    SBA          : longword;
    Pointer      : integer;
    k            : longword;

  //wandelt einstelligen HEX-String zu integer
  // strtoint('$'+ch) ist wohl gleichwertig
  function ch2hex(ch:char):integer;
  begin
    ch2hex:=ord('0'); //fail save
    if ch in ['0'..'9'] then ch2hex:=ord(ch)-ord('0');
    if ch in ['A'..'F'] then ch2hex:=ord(ch)-ord('A')+10;
    if ch in ['a'..'f'] then ch2hex:=ord(ch)-ord('a')+10;
  end;

  //liest 2stelligen HEX-String und wandelt ihn in byte
  function ReadByte:byte;
  var ch_a,ch_b : char;
      i_a,i_b   : byte;
  begin
    read(Hexfile,ch_a,ch_b);
    i_a:=ch2hex(ch_a);
    i_b:=ch2hex(ch_b);
    ReadByte:=i_a*16+i_b;
  end; (*function*)

  //liest vierstelligen HEX-String und wandelt ihn in word
  // 16 Bit stehen im File in High-Low-Reihenfolge !!!
  function ReadWord:word;
  var w_low,w_high : word;
  begin
    w_high :=ReadByte;
    w_low  :=ReadByte;
    ReadWord:=w_high*256+w_low;
  end; (*function*)

begin  //Openhexfile16
  EndFlash:=0;
  EndEE:=-1;
  HexConfig:=false; // vielleicht keine Config im file
  assignFile(Hexfile, FName);
  reset(Hexfile);
  ULBA:=0;
  SBA:=0;
  repeat                    (*lesen einer Text-Zeile*)
    read(HexFile,ch);       (* ':' *)
    ByteAnzahl  :=ReadByte;
    Adresse     :=ReadWord;
    HexTyp      :=ReadByte;
    case HexTyp of
      0:  //Data Record
      begin
        for k:=0 to (ByteAnzahl-1) do begin
          Pointer:=ULBA*$10000+SBA*$10+Adresse+k;
          //Daten nach Hexin schreiben
          case pointer of
          $000000..$01FFFF : Hexin.Flash[pointer]:=ReadByte;
          $200000..$200007 : Hexin.ID[pointer-$200000]:=ReadByte;
          $300000..$30000D : if not NoConfig then begin
                               Hexin.Config[pointer-$300000]:=ReadByte;
                               HexConfig:=true;
                             end;
          $F00000..$F003FF : Hexin.ROM[pointer-$F00000]:=ReadByte;
          end; //case
          //Belegungsmaske festlegen
          case pointer of
          $000000..$01FFFF : Maske.Flash[pointer]:=0;
          $200000..$200007 : Maske.ID[pointer-$200000]:=0;
          $300000..$30000D : if not NoConfig then Maske.Config[pointer-$300000]:=0;
          $F00000..$F003FF : Maske.ROM[pointer-$F00000]:=0;
          end; //case
          //Bestimmung der hchsten benutzten Adresse im Flash und EEPROM
          case pointer of
          $000000..$01FFFF : if pointer>EndFlash then EndFlash:=pointer;
          $F00000..$F003FF : if (pointer and $3FF)>EndEE
                               then EndEE:=(pointer and $3FF);
          end; //case
        end; //for
      end;
      2: SBA:=ReadWord;  //Extended Segment Address Record
      4: ULBA:=ReadWord;  //Extended Linear Address Record
      1:; //end of file record, wird spter ausgewertet
      else Memo18.Lines.Add('## error in HEX-file, type: '+inttostr(HexTyp));
    end; //case;
    readln(HexFile); //nchste Zeile
  until HexTyp=1;

  closefile(Hexfile);
  Memo18.Lines.Add('-- HEX-file loaded');
  Memo18.Lines.Add('-- '+inttostr(1+((EndFlash) div 1024))+' k FLASH needed');
  Memo18.Lines.Add('-- '+inttostr(EndEE+1)+' Byte EEPROM needed');

  if EndFlash>MaxFlash then Memo18.Lines.Add('## code don''t fits into FLASH');
  if EndEE>MaxEE       then Memo18.Lines.Add('## data don''t fits into EEPROM');

  if HexConfig then Memo18.Lines.Add('-- HEX-file contains Config-data');
  Memo18.Lines.Add('');
end; //openhexfile16


//prfen ob gert angeschlossen ist
procedure Tbootloader.CheckForUsbBrenner;
begin
  Firmware     := 0;
  BootloaderDa := false;
  brennerda    := true;
  brennerda    := USBinfo;          // fragt den Treiber nach vid_pid-Gerten
  if fakeb8CheckBox.checked then brennerda := true;     //debug

  if brennerda then begin
    send_buf[0]:=READ_VERSION;    //0
    Sende_Empfange(1, 4);

    //USB4all ?
    if ((receive_buf[0] =  READ_VERSION) and (receive_buf[3]=2)) or fakeTBCheckBox.checked then begin
      Button3.Visible:=true;
    end;

    //Bootloader ?
    if ((receive_buf[0] =  READ_VERSION) and (receive_buf[3]=1)) or fakeBLCheckBox.checked then begin
      BootloaderDa:=true;
      if fakeBLCheckBox.checked then  receive_buf[2]:=2;    //debug
      memo18.lines.add('-- Bootloader, V. '+ IntToStr(receive_buf[2]));
      if receive_buf[2]=5 then TargetRadioGroup.Itemindex:=1; //USB4all -> 2455
    end;

  end; //if brennerda
  BootGroupBox.visible:= BootloaderDa
end;  //CheckForUsbBrenner


// 1. upload firmware
procedure Tbootloader.Button45Click(Sender: TObject);
var k, l       : integer;
    Adresse    : integer;
    Endadresse : integer;
    Fehler     : integer;
begin
  //File laden
  Kern:=16;
  Memo18.Lines.Add('');
  OpenDialog1.Filter:='*.hex';
  OpenDialog1.Title:='load HEX-File';
  OpenDialog1.FileName:='*.hex';
//  OpenDialog1.InitialDir:=hexPfad;
  if OpenDialog1.Execute then   { Dialog zum Dateiffnen anzeigen }
    FName:=OpenDialog1.FileName;    { Datei ausgewhlt }
  if fileexists(FName) then hexpfad:=ExtractFilePath(FName);
  StatusBar1.Panels[0].Text:=ExtractFileName(FName);
  Memo18.Lines.Add('HEX-file-name : '+ExtractFileName(FName));
  LoadFile;

  if not FileFlag then exit;
  Memo18.lines.add('');
  Memo18.lines.add('hex-File selected');
  Memo18.lines.add('start to flash the control PIC');

  //brennen
  Adresse    := $800;
  Endadresse :=$7FFF;
  case TargetRadioGroup.ItemIndex  of
    0:  Endadresse :=$7FFF;   //18F2550
    1:  Endadresse :=$5FFF;   //18F2455
  end;
  BootGauge.ForeColor := clBlue;
  BootGauge.visible:=true;
  BootGauge.Progress:=0;
  while Adresse<Endadresse do begin
    BootGauge.Progress:= round( (Adresse/Endadresse*100));
    BootGauge.update;
    //64 byte lschen
    send_buf[0]:= bERASE_FLASH;
    send_buf[1]:= 1;                             // 1 x 64 byte
    send_buf[2]:=  Adresse and $0000FF;          // low
    send_buf[3]:= (Adresse and $00FF00) shr  8;  // high
    send_buf[4]:= (Adresse and $FF0000) shr 16;  // upper
    Sende_Empfange(5,1);
    //4 x 16 byte schreiben
    for k:=0 to 3 do begin
      // The write holding register for the 18F4550 family is
      // actually 32-byte. The code below only tries to write
      // 16-byte because the GUI program only sends out 16-byte
      // at a time.
      // This limitation will be fixed in the future version.
      // LEN = # of byte to write
      send_buf[0]:= bWRITE_FLASH;
      send_buf[1]:= 16;                            // lnge 16 Byte
      send_buf[2]:=  Adresse and $0000FF;          // low
      send_buf[3]:= (Adresse and $00FF00) shr  8;  // high
      send_buf[4]:= (Adresse and $FF0000) shr 16;  // upper
      for L:=0 to 15 do  send_buf[5+L]:=Hexin.Flash[Adresse+L] and $FF;
      Sende_Empfange(21,1);
      Adresse:=Adresse+16
    end;
  end;

  //prfen
  Memo18.lines.add('');
  Memo18.lines.add('check for correctnes');
  Fehler:=0;
  Adresse    := $800;
  Endadresse :=$7FFF;
  case TargetRadioGroup.ItemIndex  of
    0:  Endadresse :=$7FFF;   //18F2550
    1:  Endadresse :=$5FFF;   //18F2455
  end;
  BootGauge.Progress:=0;
  BootGauge.ForeColor := clGreen;
  while Adresse<Endadresse do begin
    BootGauge.Progress:= round( (Adresse/Endadresse*100));
    BootGauge.update;
    send_buf[0]:= bREAD_FLASH;
    send_buf[1]:= 16;                             // lnge
    send_buf[2]:=  Adresse and $0000FF;           // low
    send_buf[3]:= (Adresse and $00FF00) shr  8;   // high
    send_buf[4]:= (Adresse and $FF0000) shr 16;   // upper
    Sende_Empfange(5,send_buf[1]+5);

    for k:=0 to receive_buf[1]-1 do begin
      if (receive_buf[k+5] and $FF) <> (Hexin.Flash[Adresse+k] and $FF) then begin
        inc(Fehler);
        BootGauge.ForeColor := clRed;
      end;
    end;
    Adresse:=Adresse+16
  end;
  BootGauge.visible:=false;

  Memo18.lines.add('');
  if Fehler=0 then begin
    // Bootloader deaktivieren
    // EEPROM-Zelle 0xFE (und 0xFF) mit 0 beschreiben
    send_buf[0]:= bWRITE_EEDATA;
    send_buf[1]:= 2;            // lnge 1 Byte
    send_buf[2]:= $FE;          // low
    send_buf[3]:= 0;            // high
    send_buf[4]:= 0;            // upper
    send_buf[5]:= 0;            // 1. Byte fr 0xFE
    send_buf[6]:= 0;            // 2. Byte fr 0xFF
    Sende_Empfange(7,1);

    Memo18.lines.add('OK');
    Memo18.lines.add('now remove the jumper (if there is one)');
    Memo18.lines.add('then press the "RESET" button');
  end else Memo18.lines.add('Flash-Error');
end; // 1. upload firmware


// 2. Reset
procedure Tbootloader.Button44Click(Sender: TObject);
begin
  send_buf[0]:= bRESET_DEVICE;
  Sende_Empfange(1,0);
  memo18.lines.add('');
  Memo18.lines.add(' ');
  Memo18.lines.add('finished');
end; // 2. RESET


procedure Tbootloader.ActBootloader(Sender: TObject);
begin
  //kein Debug
  fakeb8CheckBox.visible:=false;
  fakeBLCheckBox.visible:=false;

  memo18.lines.add('');
  info(' ');
  CheckForUsbBrenner;
end;


procedure Tbootloader.FormCreate(Sender: TObject);
begin
  //kein Debug
  // die nachfolgenden Zeilen blenden Debug-Felder aus
  fakeb8CheckBox.visible:=false;
  fakeBLCheckBox.visible:=false;
  fakeTBCheckBox.visible:=false;
  Button4.visible:=false;
  //end eDEBUG
  
  memo18.lines.add('');
  info(' ');
  Button3.Visible:= false;
  if bREAD_FLASH<>1 then caption:=caption+'  DM-Version';
  CheckForUsbBrenner;
end;


procedure Tbootloader.Button1Click(Sender: TObject);
begin
  CheckForUsbBrenner;
end;

// End
procedure Tbootloader.Button2Click(Sender: TObject);
begin
  close;
end;  // End

procedure Tbootloader.fakeBLCheckBoxClick(Sender: TObject);
begin
  if  fakeBLCheckBox.Checked then fakeBLCheckBox.visible:= false;
end;

procedure Tbootloader.fakeb8CheckBoxClick(Sender: TObject);
begin
  if  fakeb8CheckBox.Checked then fakeb8CheckBox.visible:= false;
  Button4.visible:=false;
end;

procedure Tbootloader.Button3Click(Sender: TObject);
begin
  Button3.enabled:=false;
  //kennbyte loeschen
  send_buf[0]:=SYS_EEPROM;// EE
  send_buf[1]:=2;         // schreiben
  send_buf[2]:=$FE;       // Adresse
  send_buf[3]:=$FF;       // 0xFF
  Sende_Empfange(16, 16);

  //reset
  send_buf[0]:= SYS_RESET;  //bRESET_DEVICE;
  Sende_Empfange(1,0);

  sleep(2000);
  Button3.enabled:=true;
end;

procedure Tbootloader.fakeTBCheckBoxClick(Sender: TObject);
begin
  if  fakeTBCheckBox.Checked then fakeTBCheckBox.visible:= false;
end;

procedure Tbootloader.Button4Click(Sender: TObject);
begin
    // Bootloader deaktivieren
    // EEPROM-Zelle 0xFE (und 0xFF) mit 0 beschreiben
    send_buf[0]:= bWRITE_EEDATA;
    send_buf[1]:= 2;            // lnge 1 Byte
    send_buf[2]:= $FE;          // low
    send_buf[3]:= 0;            // high
    send_buf[4]:= 0;            // upper
    send_buf[5]:= 0;            // 1. Byte fr 0xFE
    send_buf[6]:= 0;            // 2. Byte fr 0xFF
    Sende_Empfange(7,1);

    Memo18.lines.add('OK');
    Memo18.lines.add('now remove the jumper (if there is one)');
    Memo18.lines.add('then press the "RESET" button');
end;

end.
