{-----------------------------------------------------------------------------------
Diese Unit ist Teil des Programms USBoot.

Das Programm USBoot ist ein einfaches Programm zum Laden eines HEX-Files
in einen PIC18Fxxxx mit USB-Schnittstelle und USB-Bootloader.
Es bentigt den Bootloader von:
http://sprut.de/electronic/pic/8bit/18f/programm/usbbootloader/usbboot.htm#bootloader
oder den Bootloader 1.0 aus dem USBurn-Paket:
http://sprut.de/electronic/soft/usburn/usburn.htm#download

Nach nderung des vid_pid-Wertes in usbdll.pas kann auch der Microchip USB-Bootloader verwendet werden:
http://www.microchip.com/stellent/idcplg?IdcService=SS_GET_PAGE&nodeId=2124&param=en022627


Copyright (C) 2006  Joerg Bredendiek
 Homepage: http:www.sprut.de
 E-Mail:   sprut@sprut.de

This program is free software; you can redistribute it and/or modify it under the terms
of the GNU General Public License as published by the Free Software Foundation;
either version 2 of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
See the GNU General Public License for more details.

You should have received a copy of the GNU General Public License along with this program;
if not, write to the Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.


Auf Deutsch:
Dieses Programm ist freie Software. Sie knnen es unter den Bedingungen
der GNU General Public License, wie von der Free Software Foundation verffentlicht,
weitergeben und/oder modifizieren, entweder gem Version 2 der Lizenz oder
(nach Ihrer Option) jeder spteren Version.

Die Verffentlichung dieses Programms erfolgt in der Hoffnung, da es Ihnen von Nutzen
sein wird, aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite Garantie der MARKTREIFE
oder der VERWENDBARKEIT FR EINEN BESTIMMTEN ZWECK. Details finden Sie in der
GNU General Public License.

Sie sollten eine Kopie der GNU General Public License zusammen mit diesem Programm erhalten haben.
Falls nicht, schreiben Sie an die Free Software Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.

--------------------------------------------------------------------------------------}

unit usbdll;

interface

uses
  Windows, Dialogs, Sysutils;

//based on http://www.sixca.com/delphi/article/microchip_usb.html

Const
  MAXSIZE           = 64;
  MPUSB_FAIL        = 0;
  MPUSB_SUCCESS     = 1;
  MP_WRITE:DWORD    = 0;
  MP_READ:DWORD     = 1;
  MAX_NUM_MPUSB_DEV = 127;

  READ_VERSION    = $00;
  // konstanten fr bootloader
  bREAD_FLASH      = $01;
//  bREAD_FLASH      = $65;   // fuer DM-Version
  bWRITE_FLASH     = $02;
  bERASE_FLASH     = $03;
  bREAD_EEDATA     = $04;
  bWRITE_EEDATA    = $05;
  bREAD_CONFIG     = $06;
  bWRITE_CONFIG    = $07;
  bEXITBOOTLOADER  = $08;
  bUPDATE_LED      = $32;
  bRESET_DEVICE    = $FF;

  SYS_EEPROM	  = $5A;
  SYS_RESET       = $FF;  


type
  DWORD  = LongInt;
  PCHAR8 = array[0..MAXSIZE] of char;
  PBYTE  = array[0..MAXSIZE] of BYTE;
  PDWORD = array[0..MAXSIZE] of DWORD;
  PVOID  = Pointer;
  UINT   = Cardinal;


var
//  vid_pid     : PCHAR8 = 'vid_04d8&pid_000c';     // microchip
  vid_pid     : PCHAR8 = 'vid_04d8&pid_ff0b';   // sprut
//  vid_pid     : PCHAR8 = 'vid_04d8&pid_fe31';     // Stefan  Dreyer
  out_pipe    : PCHAR8 = '\MCHP_EP1';
  in_pipe     : PCHAR8 = '\MCHP_EP1';
  myOutPipe   : THANDLE;
  myInPipe    : THANDLE;
  isConnected : boolean;
  send_buf    : PBYTE;
  receive_buf : PBYTE;


// Version der mpusbapi.dll auslesen
function _MPUSBGetDLLVersion():DWORD; 
             stdcall;external 'mpusbapi.dll';

// Anzahl der zu vid&pid passenden USB-Devices
function _MPUSBGetDeviceCount(pVID_PID:PCHAR8):DWORD; 
             stdcall;external 'mpusbapi.dll';

function _MPUSBOpen(instance:DWORD;pVID_PID:PCHAR8;
             pEP:PCHAR8;dwDir:DWORD;dwReserved:DWORD):
             THANDLE;stdcall;external 'mpusbapi.dll';

function _MPUSBClose(handle:THANDLE):DWORD; 
             stdcall;external 'mpusbapi.dll';

function _MPUSBRead(handle:THANDLE;var pData:PBYTE;
             dwLen:DWORD;var pLength:DWORD;
             dwMilliseconds:DWORD):DWORD;stdcall;
             external 'mpusbapi.dll';

function _MPUSBReadInt(handle:THANDLE;
             var pData:PBYTE;dwLen:DWORD;
             var pLength:PDWORD;
             dwMilliseconds:DWORD):DWORD; 
             stdcall;external 'mpusbapi.dll';

function _MPUSBWrite(handle:THANDLE;pData:PBYTE;
             dwLen:DWORD;
             var pLength:DWORD;
             dwMilliseconds:DWORD):DWORD; 
             stdcall;external 'mpusbapi.dll';

function SendReceivePacket(SendData:PBYTE;SendLength:DWORD;var ReceiveData:PBYTE;
         var ReceiveLength:DWORD;SendDelay:UINT;ReceiveDelay:UINT):DWORD; stdcall;

implementation


procedure CheckInvalidHandle();
begin
  if(GetLastError=ERROR_INVALID_HANDLE) then
  begin
        _MPUSBClose(myOutPipe);
        _MPUSBClose(myInPipe);

        myInPipe:=INVALID_HANDLE_VALUE;
        myOutPipe:=INVALID_HANDLE_VALUE;
  end
  else
    ShowMessage('Error Code :'+inttostr(GetLastError()));
end;



function SendReceivePacket(SendData:PBYTE;SendLength:DWORD;var ReceiveData:PBYTE;
         var ReceiveLength:DWORD;SendDelay:UINT;ReceiveDelay:UINT):DWORD; stdcall;
var
  SentDataLength:DWORD ;
  ExpectedReceiveLength:DWORD;
begin
// Fehlercodes:
//   0 -   keine outpipe bekommen
//   1 -   alles ok
//   2 -   falsche Anzahl empfangen
// 100 -   schreiben oder lesen ging nicht
  result:=100;
  ExpectedReceiveLength:= ReceiveLength;
  if((myOutPipe <> INVALID_HANDLE_VALUE) and (myInPipe <> INVALID_HANDLE_VALUE)) then begin
    if(_MPUSBWrite(myOutPipe,SendData,SendLength,SentDataLength,SendDelay)<>0) then

    if(_MPUSBRead(myInPipe,ReceiveData,ExpectedReceiveLength,ReceiveLength,ReceiveDelay)<>0)then begin

      if(ReceiveLength = ExpectedReceiveLength) then begin
        Result:=1; // Success
        exit;
      end else //ReceiveLength
      if(ReceiveLength < ExpectedReceiveLength) then begin
        Result:=2;     // incorrect receive length
        exit;
      end

    end else          //_MPUSBRead
    CheckInvalidHandle()
    else              //_MPUSBWrite
    CheckInvalidHandle()
  end else begin      //myOutPipe
    Result:=0;        // Failed
  end;                //myOutPipe
end;


initialization


end.
